/* ****************************************************************************
   **                                                                        **
   **        Calcul du "Compte est bon" des Chiffres et des lettres          **
   **      (c) Copyright 16-06-1998  par FlashCode (Sebastien HELLEU)        **
   **                                                                        **
   **                        http://www.flashtux.org                         **
   **                        e-mail:flashcode@flashtux.org                   **
   **                                                                        **
   ****************************************************************************

==== This program is free software; you can redistribute it and/or modify
==== it under the terms of the GNU General Public License as published by
==== the Free Software Foundation; either version 2 of the License, or
==== (at your option) any later version.
====
==== This program is distributed in the hope that it will be useful,
==== but WITHOUT ANY WARRANTY; without even the implied warranty of
==== MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
==== GNU General Public License for more details.
====
==== You should have received a copy of the GNU General Public License
==== along with this program; if not, write to the Free Software
==== Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

Auteur  : FlashCode <flashcode@flashtux.org>
Creation: 16/06/1998

26/08/2001: FlashCode:
    - ajout de l'aide
    - ajout de la licence
    - reecriture du source (plus comprehensible)
    - correction bug affichage resultat quand plaques passees par ligne de commande
*/

static char id[] = "$Id: CHIFFRES (c) Copyright 2001/08/26 by FlashCode <flashcode@free.fr> $";


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define OP_ADDITION         1
#define OP_SOUSTRACTION     2
#define OP_MULTIPLICATION   3
#define OP_DIVISION         4


int     tableau_plaques[14] = { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 25, 50, 75, 100 };
int     resultat, plaque[6], sauve_plaque[6];
int     meilleur_resultat, meilleur_difference;
int     solution[30], indice_solution;


/*
 * affiche_aide: affiche l'aide sur Chiffres
 */

void affiche_aide()
{
    printf("\nChiffres (c) Copyright 26/08/2001 par FlashCode (Sebastien Helleu)\n\n");
    printf("Syntaxe:  chiffres [-?] [p1 p2 p3 p4 p5 p6 res] [-licence]\n\n");
    printf("     -?        affiche cet ecran d'aide\n");
    printf("  p1..p6 res   utilise les plaques p1 a p6 et le resultat res fournis\n");
    printf("   -licence    affiche la licence de Chiffres\n\n");
    exit(0);
}

/*
 * affiche_licence: affiche la licence de Chiffres
 */

void affiche_licence()
{
    printf("\nCHIFFRES  (c) Copyright 1998,1999,2000,2001 by FlashCode (Sebastien Helleu)\n\n");
    printf("This program is free software; you can redistribute it and/or modify\n");
    printf("it under the terms of the GNU General Public License as published by\n");
    printf("the Free Software Foundation; either version 2 of the License, or\n");
    printf("(at your option) any later version.\n");
    printf("\n");
    printf("This program is distributed in the hope that it will be useful,\n");
    printf("but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
    printf("MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
    printf("GNU General Public License for more details.\n");
    printf("\n");
    printf("You should have received a copy of the GNU General Public License\n");
    printf("along with this program; if not, write to the Free Software\n");
    printf("Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA\n\n");
    exit(0);
}

/*
 * choisit_chiffres: tire les plaques au hasard
 */

void choisit_chiffres()
{
    int     i;

    srand((unsigned)(time(NULL)));
    for (i = 0; i < 6; i++)
    {
        plaque[i] = tableau_plaques[rand()%14];
        sauve_plaque[i] = plaque[i];
    }
    resultat = rand()%900+100;
}

/*
 * affiche_chiffres: affiche les six plaques
 */

void affiche_chiffres(int chiffre[], unsigned char *chaine)
{
    unsigned char   i;

    for (i = 0; i < 6; i++)
        printf("%d  ", chiffre[i]);
    printf("/  %d", resultat);
    if (chaine != NULL)
        printf("%s", chaine);
    printf("\n");
}

/*
 * initialise_variables: init variables au debut
 */

void initialise_variables()
{
    int     i;

    meilleur_resultat = 0;
    meilleur_difference = resultat;
    indice_solution = 0;
    for (i = 0; i < 30; i++)
        solution[i] = 0;
}

/*
 * affiche_solution: affiche la solution
 */

void affiche_solution()
{
    int     i;

    if (meilleur_resultat == resultat)
    {
        printf("##### SOLUTION pour:  ");
        affiche_chiffres(sauve_plaque, "  #####");
    }
    printf("Res %d: ", meilleur_resultat);
    i = 0;
    while (solution[i])
    {
        printf("%d", solution[i]);
        switch (solution[i+2])
        {
            case OP_ADDITION:printf("+"); break;
            case OP_SOUSTRACTION:printf("-"); break;
            case OP_MULTIPLICATION:printf("*"); break;
            case OP_DIVISION:printf("/"); break;
        }
        printf("%d=", solution[i+1]);
        switch (solution[i+2])
        {
            case OP_ADDITION:printf("%d  ", solution[i]+solution[i+1]); break;
            case OP_SOUSTRACTION:printf("%d  ", solution[i]-solution[i+1]); break;
            case OP_MULTIPLICATION:printf("%d  ", solution[i]*solution[i+1]); break;
            case OP_DIVISION:printf("%d  ", solution[i]/solution[i+1]); break;
        }
        i += 3;
    }
    printf("\n");
}

/*
 * cherche_resultat: fonction recursive qui cherche le meilleur resultat possible
 */

void cherche_resultat()
{
    int     i, j, temp1, temp2;

    for (i = 0; i < 6; i++)
        if (abs(resultat-plaque[i]) < meilleur_difference)
        {
            meilleur_resultat = plaque[i];
            meilleur_difference = abs(resultat-plaque[i]);
            affiche_solution();
            if (meilleur_resultat == resultat) exit(0);
        }
    for (i = 0; i < 6; i++)
        if (plaque[i] != 0)
            for (j = i+1; j < 6; j++)
                if (plaque[j] != 0)
                {
                    temp1 = plaque[i];
                    temp2 = plaque[j];
                    solution[indice_solution] = temp1;
                    solution[indice_solution+1] = temp2;
                    indice_solution += 3;
                    plaque[i] = plaque[i]+plaque[j];
                    plaque[j] = 0;
                    solution[indice_solution-1] = OP_ADDITION;
                    cherche_resultat();
                    plaque[i] = temp1;
                    plaque[j] = temp2;
                    if (plaque[i] > plaque[j])
                    {
                        plaque[i] = plaque[i]-plaque[j];
                        plaque[j] = 0;
                        solution[indice_solution-1] = OP_SOUSTRACTION;
                        cherche_resultat();
                        plaque[i] = temp1;
                        plaque[j] = temp2;
                    }
                    else
                        if (plaque[j] > plaque[i])
                        {
                            plaque[i] = plaque[j]-plaque[i];
                            plaque[j] = 0;
                            solution[indice_solution-3] = temp2;
                            solution[indice_solution-2] = temp1;
                            solution[indice_solution-1] = OP_SOUSTRACTION;
                            cherche_resultat();
                            plaque[i] = temp1;
                            plaque[j] = temp2;
                        }
                    plaque[i] = plaque[i]*plaque[j];
                    plaque[j] = 0;
                    solution[indice_solution-1] = OP_MULTIPLICATION;
                    cherche_resultat();
                    plaque[i] = temp1;
                    plaque[j] = temp2;
                    if (plaque[i] >= plaque[j])
                    {
                        if (plaque[i]%plaque[j] == 0)
                        {
                            plaque[i] = plaque[i]/plaque[j];
                            plaque[j] = 0;
                            solution[indice_solution-1] = OP_DIVISION;
                            cherche_resultat();
                            plaque[i] = temp1;
                            plaque[j] = temp2;
                        }
                    }
                    else
                        if (plaque[j]%plaque[i] == 0)
                        {
                            plaque[i] = plaque[j]/plaque[i];
                            plaque[j] = 0;
                            solution[indice_solution-3] = temp2;
                            solution[indice_solution-2] = temp1;
                            solution[indice_solution-1] = OP_DIVISION;
                            cherche_resultat();
                            plaque[i] = temp1;
                            plaque[j] = temp2;
                        }
                    indice_solution -= 3;
                    solution[indice_solution] = 0;
                }
}

int main(int argc, char *argv[])
{
    int     i;

    if (argc > 1)
    {
        if (strcmp(argv[1], "-?") == 0)
            affiche_aide();
        if (strcasecmp(argv[1], "-licence") == 0)
            affiche_licence();
    }
    if (argc == 8)
    {
        for (i = 0; i < 6; i++)
        {
            plaque[i] = atoi(argv[i+1]);
            sauve_plaque[i] = plaque[i];
        }
        resultat = atoi(argv[7]);
    }
    else
        choisit_chiffres();
    affiche_chiffres(plaque, NULL);
    initialise_variables();
    cherche_resultat();
    return 0;
}
